#lang racket
(require "run.rkt"
         "status.rkt"
         racket/port
         syntax/wrap-modbeg
         (for-syntax syntax/parse))

(provide (rename-out [pfsh:module-begin #%module-begin]
                     [pfsh:run run]
                     [pfsh:define define])
         &&)

(define-syntax pfsh:module-begin
  (make-wrapping-module-begin #'status-check-error))

(define (status-check-error v)
  (when (status-error? v) (exit)))

(begin-for-syntax
  (define-syntax-class run-arg
    (pattern sym:id
             #:with expr #`#,(symbol->string (syntax-e #'sym)))
    (pattern str:string
             #:with expr #'(quote str))))

(define-syntax (pfsh:run stx)
  (syntax-parse stx
    #:datum-literals (<)
    [(_ prog:run-arg arg:run-arg ... < stream:id)
     #'(with-input-from-string
         stream
         (lambda ()
           (pfsh:run prog arg ...)))]
    [(_ prog:run-arg arg:run-arg ...)
     #`(boolean->status (run prog.expr arg.expr ...))]))

(define-syntax (pfsh:define stx)
  (syntax-parse stx
    [(_ stream:id expr)
     #'(define stream (with-output-to-string
                        (lambda ()
                          expr)))]))

(define-syntax (&& stx)
  (syntax-parse stx
    [(_ e1 e2)
     #`(if (status-failure? e1)
           failure-status ; not an error, even if e1 was an error
           e2)]))
